#!/usr/bin/perl

package Search::GIN;

use strict;
use warnings;

our $VERSION = "0.02";

__PACKAGE__

__END__

=pod

=head1 NAME

Search::GIN - Generalized Inverted Indexing

=head1 SYNOPSIS

	# Coming Soon

=head1 DESCRIPTION

This is used by L<KiokuDB> for custom indexing.

More documentation will be released shortly.

=cut


